@@ -4,17 +4,6 @@ import pytest
 import sys
 from common import collect, skip_on_pypy
 
-# Some helper declaration to check types across different Python versions
-if sys.version_info < (3, 9):
-    TYPING_DICT = "typing.Dict"
-    TYPING_SET = "typing.Set"
-    TYPING_ABC = "typing"
-    TYPING_ABSTRACT_SET = "typing.AbstractSet"
-else:
-    TYPING_DICT = "dict"
-    TYPING_SET = "set"
-    TYPING_ABC = "collections.abc"
-    TYPING_ABSTRACT_SET = "collections.abc.Set"
 
 
 def optional(arg: str, /) -> str:
@@ -502,13 +491,22 @@ def test49_std_variant_unbound_type():
         f"variant_unbound_type(x: {arg_t} = None) -> {rv_t}"
     )
 
+def test49b_std_variant_nondefault(clean):
+    assert t.variant_nondefault(t.NonDefaultConstructible(10)) == 10
+    assert t.variant_nondefault(20) == -20
+    assert_stats(
+        value_constructed=1, # constructed in NonDefaultConstructible pyobject
+        copy_constructed=1, # copied into type_caster variant member
+        move_constructed=1, # moved into function argument value
+        destructed=3,
+    )
 
 def test50_map_return_movable_value():
     for i, (k, v) in enumerate(sorted(t.map_return_movable_value().items())):
         assert k == chr(ord("a") + i)
         assert v.value == i
         assert t.map_return_movable_value.__doc__ == (
-            f"map_return_movable_value() -> {TYPING_DICT}[str, test_stl_ext.Movable]"
+            "map_return_movable_value() -> dict[str, test_stl_ext.Movable]"
         )
 
 
@@ -517,14 +515,14 @@ def test51_map_return_copyable_value():
         assert k == chr(ord("a") + i)
         assert v.value == i
         assert t.map_return_copyable_value.__doc__ == (
-            f"map_return_copyable_value() -> {TYPING_DICT}[str, test_stl_ext.Copyable]"
+            "map_return_copyable_value() -> dict[str, test_stl_ext.Copyable]"
         )
 
 
 def test52_map_movable_in_value():
     t.map_movable_in_value(dict([(chr(ord("a") + i), t.Movable(i)) for i in range(10)]))
     assert t.map_movable_in_value.__doc__ == (
-        f"map_movable_in_value(x: {TYPING_ABC}.Mapping[str, test_stl_ext.Movable]) -> None"
+        "map_movable_in_value(x: collections.abc.Mapping[str, test_stl_ext.Movable]) -> None"
     )
 
 
@@ -533,7 +531,7 @@ def test53_map_copyable_in_value():
         dict([(chr(ord("a") + i), t.Copyable(i)) for i in range(10)])
     )
     assert t.map_copyable_in_value.__doc__ == (
-        f"map_copyable_in_value(x: {TYPING_ABC}.Mapping[str, test_stl_ext.Copyable]) -> None"
+        "map_copyable_in_value(x: collections.abc.Mapping[str, test_stl_ext.Copyable]) -> None"
     )
 
 
@@ -542,7 +540,7 @@ def test54_map_movable_in_lvalue_ref():
         dict([(chr(ord("a") + i), t.Movable(i)) for i in range(10)])
     )
     assert t.map_movable_in_lvalue_ref.__doc__ == (
-        f"map_movable_in_lvalue_ref(x: {TYPING_ABC}.Mapping[str, test_stl_ext.Movable]) -> None"
+        "map_movable_in_lvalue_ref(x: collections.abc.Mapping[str, test_stl_ext.Movable]) -> None"
     )
 
 
@@ -551,14 +549,14 @@ def test55_map_movable_in_rvalue_ref():
         dict([(chr(ord("a") + i), t.Movable(i)) for i in range(10)])
     )
     assert t.map_movable_in_rvalue_ref.__doc__ == (
-        f"map_movable_in_rvalue_ref(x: {TYPING_ABC}.Mapping[str, test_stl_ext.Movable]) -> None"
+        "map_movable_in_rvalue_ref(x: collections.abc.Mapping[str, test_stl_ext.Movable]) -> None"
     )
 
 
 def test56_map_movable_in_ptr():
     t.map_movable_in_ptr(dict([(chr(ord("a") + i), t.Movable(i)) for i in range(10)]))
     assert t.map_movable_in_ptr.__doc__ == (
-        f"map_movable_in_ptr(x: {TYPING_ABC}.Mapping[str, test_stl_ext.Movable]) -> None"
+        "map_movable_in_ptr(x: collections.abc.Mapping[str, test_stl_ext.Movable]) -> None"
     )
 
 
@@ -592,9 +590,9 @@ def test60_set_return_value():
         assert k == chr(ord("a") + i)
     for i, k in enumerate(sorted(t.unordered_set_return_value())):
         assert k == chr(ord("a") + i)
-    assert t.set_return_value.__doc__ == (f"set_return_value() -> {TYPING_SET}[str]")
+    assert t.set_return_value.__doc__ == "set_return_value() -> set[str]"
     assert t.unordered_set_return_value.__doc__ == (
-        f"unordered_set_return_value() -> {TYPING_SET}[str]"
+        "unordered_set_return_value() -> set[str]"
     )
 
 
@@ -602,24 +600,24 @@ def test61_set_in_value():
     t.set_in_value(set([chr(ord("a") + i) for i in range(10)]))
     t.unordered_set_in_value(set([chr(ord("a") + i) for i in range(10)]))
     assert t.set_in_value.__doc__ == (
-        f"set_in_value(x: {TYPING_ABSTRACT_SET}[str]) -> None"
+        "set_in_value(x: collections.abc.Set[str]) -> None"
     )
     assert t.unordered_set_in_value.__doc__ == (
-        f"unordered_set_in_value(x: {TYPING_ABSTRACT_SET}[str]) -> None"
+        "unordered_set_in_value(x: collections.abc.Set[str]) -> None"
     )
 
 
 def test62_set_in_lvalue_ref():
     t.set_in_lvalue_ref(set([chr(ord("a") + i) for i in range(10)]))
     assert t.set_in_lvalue_ref.__doc__ == (
-        f"set_in_lvalue_ref(x: {TYPING_ABSTRACT_SET}[str]) -> None"
+        "set_in_lvalue_ref(x: collections.abc.Set[str]) -> None"
     )
 
 
 def test63_set_in_rvalue_ref():
     t.set_in_rvalue_ref(set([chr(ord("a") + i) for i in range(10)]))
     assert t.set_in_rvalue_ref.__doc__ == (
-        f"set_in_rvalue_ref(x: {TYPING_ABSTRACT_SET}[str]) -> None"
+        "set_in_rvalue_ref(x: collections.abc.Set[str]) -> None"
     )
 
 
@@ -686,29 +684,91 @@ def test67_vector_bool():
 
 def test68_complex_value():
     # double: 64bits
-    assert t.complex_value_double(1.0) == 1.0
+    assert t.complex_value_double(1.0) == complex(1.0)
     assert t.complex_value_double(1.0j) == 1.0j
-    assert t.complex_value_double(0.0) == 0.0
+    assert t.complex_value_double(0.0) == complex(0.0)
     assert t.complex_value_double(0.0j) == 0.0j
-    assert t.complex_value_double(0) == 0
-    assert t.complex_value_float(1.0) == 1.0
+    assert t.complex_value_double(0) == complex(0.0)
+    assert t.complex_value_double_nc(1.0 + 2.0j) == 1.0 + 2.0j
+    assert t.complex_value_double_nc(1.0j) == 1.0j
+    assert t.complex_value_double_nc(0.0 + 0.0j) == complex(0.0)
+    assert t.complex_value_double_nc(0.0j) == 0.0j
+    with pytest.raises(TypeError, match="incompatible function arguments"):
+        t.complex_value_double_nc(0)
+    with pytest.raises(TypeError, match="incompatible function arguments"):
+        t.complex_value_double_nc(0.0)
+
+    # float: 32bits
+    assert t.complex_value_float(1.0) == complex(1.0)
     assert t.complex_value_float(1.0j) == 1.0j
-    assert t.complex_value_float(0.0) == 0.0
+    assert t.complex_value_float(0.0) == complex(0.0)
     assert t.complex_value_float(0.0j) == 0.0j
-    assert t.complex_value_float(0) == 0
+    assert t.complex_value_float(0) == complex(0.0)
+    assert t.complex_value_float_nc(1.0 + 2.0j) == 1.0 + 2.0j
+    assert t.complex_value_float_nc(1.0j) == 1.0j
+    assert t.complex_value_float_nc(0.0 + 0.0j) == complex(0.0)
+    assert t.complex_value_float_nc(0.0j) == 0.0j
+    with pytest.raises(TypeError, match="incompatible function arguments"):
+        t.complex_value_float_nc(0)
+    with pytest.raises(TypeError, match="incompatible function arguments"):
+        t.complex_value_float_nc(0.0)
+    with pytest.raises(TypeError, match="incompatible function arguments"):
+        t.complex_value_float_nc(1.1 + 2.0j)  # Inexact narrowing conversion
+    with pytest.raises(TypeError, match="incompatible function arguments"):
+        t.complex_value_float_nc(1.0 + 2.1j)  # Inexact narrowing conversion
 
     val_64 = 2.7 - 3.2j
     val_32 = 2.700000047683716 - 3.200000047683716j
     assert val_64 != val_32
 
-    assert t.complex_value_float(val_32) == val_32
-    assert t.complex_value_float(val_64) == val_32
     assert t.complex_value_double(val_32) == val_32
     assert t.complex_value_double(val_64) == val_64
+    assert t.complex_value_double_nc(val_32) == val_32
+    assert t.complex_value_double_nc(val_64) == val_64
+
+    assert t.complex_value_float(val_32) == val_32
+    assert t.complex_value_float(val_64) == val_32
+    assert t.complex_value_float_nc(val_32) == val_32
+    with pytest.raises(TypeError, match="incompatible function arguments"):
+        t.complex_value_float_nc(val_64)  # Inexact narrowing conversion
 
     with pytest.raises(TypeError, match="incompatible function arguments"):
         t.complex_value_float([])
 
+    class MyInt(int):
+        def __new__(cls, value):
+            return super().__new__(cls, value)
+
+    assert t.complex_value_double(MyInt(7)) == complex(7.0)
+    assert t.complex_value_float(MyInt(7)) == complex(7.0)
+    with pytest.raises(TypeError, match="incompatible function arguments"):
+        t.complex_value_double_nc(MyInt(7))
+    with pytest.raises(TypeError, match="incompatible function arguments"):
+        t.complex_value_float_nc(MyInt(7))
+
+    class MyComplex:
+        def __init__(self, real, imag):
+            self.re = real
+            self.im = imag
+        def __complex__(self):
+            return complex(self.re, self.im)
+
+    assert t.complex_value_double(MyComplex(1, 2)) == complex(1.0, 2.0)
+    assert t.complex_value_float(MyComplex(1, 2)) == complex(1.0, 2.0)
+    with pytest.raises(TypeError, match="incompatible function arguments"):
+        t.complex_value_double_nc(MyComplex(1, 2))
+    with pytest.raises(TypeError, match="incompatible function arguments"):
+        t.complex_value_float_nc(MyComplex(1, 2))
+
+    class MyComplexSubclass(complex): ...
+
+    assert t.complex_value_double(MyComplexSubclass(1, 2)) == complex(1.0, 2.0)
+    assert t.complex_value_float(MyComplexSubclass(1, 2)) == complex(1.0, 2.0)
+    with pytest.raises(TypeError, match="incompatible function arguments"):
+        t.complex_value_double_nc(MyComplexSubclass(1, 2))
+    with pytest.raises(TypeError, match="incompatible function arguments"):
+        t.complex_value_float_nc(MyComplexSubclass(1, 2))
+
     try:
         import numpy as np
 
@@ -716,10 +776,17 @@ def test68_complex_value():
         assert t.complex_value_float(np.complex64(val_64)) == val_32
         assert t.complex_value_double(np.complex64(val_32)) == val_32
         assert t.complex_value_double(np.complex64(val_64)) == val_32
+
         assert t.complex_value_float(np.complex128(val_32)) == val_32
         assert t.complex_value_float(np.complex128(val_64)) == val_32
         assert t.complex_value_double(np.complex128(val_32)) == val_32
         assert t.complex_value_double(np.complex128(val_64)) == val_64
+
+        with pytest.raises(TypeError, match="incompatible function arguments"):
+            t.complex_value_double_nc(np.complex128(val_64))
+        with pytest.raises(TypeError, match="incompatible function arguments"):
+            t.complex_value_float_nc(np.complex64(val_32))
+
     except ImportError:
         pass
 
@@ -782,6 +849,7 @@ def test69_complex_array():
 def test70_vec_char():
     assert isinstance(t.vector_str("123"), str)
     assert isinstance(t.vector_str(["123", "345"]), list)
+    assert t.vector_optional_str(["abc", None]) == ["abc", None]
 
 
 def test71_null_input():
@@ -793,3 +861,13 @@ def test71_null_input():
 @skip_on_pypy # PyPy fails this test on Windows :-(
 def test72_wstr():
     assert t.pass_wstr('🎈') == '🎈'
+
+def test73_bad_input_to_set():
+    with pytest.raises(TypeError):
+        t.set_in_value(None)
+
+def test74_variant_implicit_conversions():
+    event = t.IDHavingEvent()
+    assert event.id is None
+    event.id = t.BasicID1(78)
+    assert type(event.id) is t.BasicID1
